import 'package:flutter/material.dart';

import 'package:flutter/services.dart';
import 'package:dvote/dvote.dart';

final messageToSign = "hello";
final privKeyToSign =
    "fad9c8855b740a0b7ed4c221dbad0f33a83a49cad6b3fe8d5817ac83d38b6a19";
final publicKey =
    "049a7df67f79246283fdc93af76d4f8cdd62c4886e8cd870944e817dd0b97934fdd7719d0810951e03418205868a5c1b40b192451367f28e0088dd75e15de40c05";

class SigningScreen extends StatefulWidget {
  @override
  _SigningScreenState createState() => _SigningScreenState();
}

class _SigningScreenState extends State<SigningScreen> {
  String _signature = "-";
  bool _validSignature = false;
  String _error;

  @override
  void initState() {
    super.initState();
    initPlatformState();
  }

  // Platform messages are asynchronous, so we initialize in an async method.
  Future<void> initPlatformState() async {
    String signature;
    bool valid;
    String error;

    // Platform messages may fail, so we use a try/catch PlatformException.
    try {
      signature = await signString(messageToSign, privKeyToSign);
      valid = await verifySignature(signature, messageToSign, publicKey);
    } on PlatformException catch (err) {
      error = err.message;
    } catch (err) {
      if (err is String)
        error = err;
      else
        error = "Failed to verify the signature";
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;

    if (error != null) {
      setState(() {
        _error = error;
      });
      return;
    }

    setState(() {
      _signature = signature;
      _validSignature = valid ?? false;
    });
  }

  @override
  Widget build(BuildContext context) {
    if (_error != null) {
      return Scaffold(
        appBar: AppBar(
          title: const Text('ECDSA Signatures'),
        ),
        body: Container(
          child: Text("Error: " + _error),
        ),
      );
    }

    final signingData = """
Signing '$messageToSign' with $privKeyToSign:
        
$_signature

The signature is ${_validSignature ? "valid" : "not valid"}""";

    return Scaffold(
      appBar: AppBar(
        title: const Text('ECDSA Signatures'),
      ),
      body: ListView(
        children: <Widget>[
          Padding(
            padding: EdgeInsets.all(16),
            child: Column(
              children: <Widget>[
                Text(signingData),
              ],
            ),
          ),
        ],
      ),
    );
  }
}
