import 'package:flutter/material.dart';

import 'package:flutter/services.dart';
import 'package:dvote/dvote.dart';
import 'package:flutter_dotenv/flutter_dotenv.dart';

class WalletScreen extends StatefulWidget {
  @override
  _WalletScreenState createState() => _WalletScreenState();
}

class _WalletScreenState extends State<WalletScreen> {
  String _mnemonic = '(mnemonic)';
  String _privKey = '(priv key)';
  String _pubKey = '(pub key)';
  String _address = '(addr)';
  String _error;

  @override
  void initState() {
    super.initState();
    initPlatformState();
  }

  // Platform messages are asynchronous, so we initialize in an async method.
  Future<void> initPlatformState() async {
    String mnemonic, path, privKey, pubKey, addr;
    String error;

    // Platform messages may fail, so we use a try/catch PlatformException.
    try {
      await DotEnv().load('.env');
      mnemonic = await generateMnemonic(size: 192);
      path = DotEnv().env['PATH'];
      privKey = await mnemonicToPrivateKey(mnemonic, hdPath: path);
      pubKey = await mnemonicToPublicKey(mnemonic, hdPath: path);
      addr = await mnemonicToAddress(mnemonic, hdPath: path);
    } on PlatformException catch (err) {
      error = err.message;
    } catch (err) {
      error = err;
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;

    if (error != null) {
      setState(() {
        _error = error;
      });
      return;
    }

    setState(() {
      _mnemonic = mnemonic;
      _privKey = privKey;
      _pubKey = pubKey;
      _address = addr;
    });
  }

  @override
  Widget build(BuildContext context) {
    if (_error != null) {
      return Scaffold(
        appBar: AppBar(
          title: const Text('HD Wallets'),
        ),
        body: Container(
          child: Text("Error: " + _error),
        ),
      );
    }

    return Scaffold(
      appBar: AppBar(
        title: const Text('HD Wallets'),
      ),
      body: ListView(
        children: <Widget>[
          Padding(
            padding: EdgeInsets.all(16),
            child: Column(
              children: <Widget>[
                Text('Mnemonic: \n$_mnemonic'),
                SizedBox(height: 15),
                Text('Private key: \n$_privKey'),
                SizedBox(height: 15),
                Text('Public key: \n$_pubKey'),
                SizedBox(height: 15),
                Text('Address: \n$_address'),
              ],
            ),
          ),
        ],
      ),
    );
  }
}
