import 'package:dvote/dvote.dart';
import '../net/gateway.dart';
import "../gomobile/hashing.dart";

/// Returns the Poseidon hash of the given hex ECDSA public key.
/// The result is provided in base64
Future<String> digestHexClaim(String hexPublicKey) {
  return poseidonHash(hexPublicKey);
}

/// Fetch the Merkle Proof that proves that the given claim is part
/// of the Census Merkle Tree with the given Root Hash
Future<String> generateProof(String censusMerkleRootHash, String base64Claim,
    DVoteGateway dvoteGw) async {
  if (!(censusMerkleRootHash is String) || !(base64Claim is String))
    throw Exception('Invalid parameters');
  try {
    Map<String, dynamic> reqParams = {
      "method": "genProof",
      "censusId": censusMerkleRootHash,
      "claimData": base64Claim,
    };
    Map<String, dynamic> response = await dvoteGw.request(reqParams);
    if (!(response is Map) || !(response["siblings"] is String)) {
      throw Exception("Invalid response received from the gateway");
    }
    return response["siblings"];
  } catch (err) {
    throw Exception("The claim proof could not be generated");
  }
}

/// Get the number of people in the census with the given Merkle Root Hash
Future<int> getCensusSize(
    String censusMerkleRootHash, DVoteGateway dvoteGw) async {
  if (!(censusMerkleRootHash is String) || !(dvoteGw is DVoteGateway))
    throw Exception('Invalid parameters');
  try {
    Map<String, dynamic> reqParams = {
      "method": "getSize",
      "censusId": censusMerkleRootHash
    };
    Map<String, dynamic> response = await dvoteGw.request(reqParams);
    if (!(response is Map) || !(response["size"] is int)) {
      throw Exception("Invalid response received from the gateway");
    }
    return response["size"];
  } catch (err) {
    throw Exception("The claim proof could not be generated");
  }
}
